/************************************************************************************/
/* Copyright (c) 2003, Outside Echo Ltd. & Hewlett-Packard.                         */
/* All rights reserved.                                                             */  
/*                                                                                  */
/* Redistribution and use in source and binary forms, with or without modification, */
/* are permitted provided that the following conditions are met:                    */
/*                                                                                  */
/*    * Redistributions of source code must retain the above copyright notice, this */
/*      list of conditions and the following disclaimer.                            */
/*    * Redistributions in binary form must reproduce the above copyright notice,   */
/*      this list of conditions and the following disclaimer in the documentation   */
/*      and/or other materials provided with the distribution.                      */
/*    * Neither the name of Outside Echo Ltd. & Hewlett-Packard                     */
/*      nor the names of its contributors may be used to endorse or promote products*/ 
/*      derived from this software without specific prior written permission.       */
/*                                                                                  */ 
/*                                                                                  */
/*   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS NAMELY     */
/*   OUTSIDE ECHO LTD AND HEWLETT-PACKARD "AS IS" AND ANY                           */
/*   EXPRESS  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/*   WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/*   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE    */
/*   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     */
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR     */
/*   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER     */
/*   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  */
/*   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  DAMAGE.          */
/*                                                                                  */
/************************************************************************************/
/* This file contains the classes for rule structure creation                       */
/*                                                                                  */
/*                                                                                  */
/* AUTHOR: Partha Pratim Talukdar, Hewlett-Packard Labs (India)                     */
/* Email: (ptpartha@india.hp.com)                                                   */
/* CREATED:  Oct 10, 2003                                                           */
/* MODIFIED: Jan 7, 2004                                                            */
/* MODIFIED: Feb, 2005 Roger Tucker - added explicit instantiation of Vector        */
/*                                    class functions for RuleBase::RuleHeadNode    */
/*                                                                                  */
/************************************************************************************/

#include "headers.h"
#include "Mapping.h"
#include "error.h"

#include "RuleBase.h"
#include "Vector.h"

// #include "vector.h"

using namespace std;

// template class Vector<RuleBase::RuleHeadNode>;
// Roger - problem in one of Vector functions prevents whole class being instantiated
// so have instantiated just the ones that are needed:

template void Vector<RuleBase::RuleHeadNode>::push_back(const RuleBase::RuleHeadNode);
template int Vector<RuleBase::RuleHeadNode>::size();
template Vector<RuleBase::RuleHeadNode>::Vector();

/* Constructor */

RuleBase::RuleBase()
{
   if((logFP = fopen("rule_log", "w")) == NULL)
      Error::PrintAndExit("Log file could not be opened");
}

/* This method does the function of matching the rules against the current input */
/* and performs necessary action                                                 */
                                   
int RuleBase::Match(char *inputWord, MappingTable mappingTable, char *ruleModifier)
{
   int i, wordIdx, startIdx;
   unsigned int chIdx;
   RuleNode   *currNode;
   bool       matched;
	
   flag = 0;
   wordIdx = 0;
   startIdx = 0;

   phoneticEq.fragmentCount = 0;  /* This count is reset for each word */

   matched = false;

   /* One character look ahead is used and hence run till one but last character */
   /* The last character is anyway processed in the process                      */

   if(PRINTLOG)
      fprintf(logFP,"\n-----------------------------------------------------\n");

   while(wordIdx <strlen(inputWord)) 
   {	   	  
      chIdx = ReturnInteger(inputWord[wordIdx]);
      ++wordIdx;

      /* printf("Working with: %s S:%d W:%d\n", mappingTable.mappingNode[chIdx].className, startIdx, wordIdx); */

      /* New rule has to be found */

      if(!matched) 
      {
	 for(i=0; i<ruleHead.size(); i++)
	 {	
	    if(ruleHead[i].head != NULL)
	    {
	       if(strcmp(ruleHead[i].head->className, mappingTable.mappingNode[chIdx].className) == 0)
	       {		
		  if(PRINTLOG)
		     fprintf(logFP,"Head:%s ", mappingTable.mappingNode[chIdx].className);

		  currNode = ruleHead[i].head;
		  matched = true;
		  break;
	       }
	    }
	 }
      }

      /* This means that the starting character class didn't match any of the rules. 
       */
      if(!matched) 
      {
	 if(PRINTLOG)
	    fprintf(logFP," #Head#:%s :: NOT APPLIED ::\n", mappingTable.mappingNode[chIdx].className);

	 StorePhoneString(chIdx, mappingTable.mappingNode[chIdx].mappedStr);
	 ++startIdx;

	 continue;
      }

      /* If this is not the character against which the head node was matched. 
       */
      if(startIdx != wordIdx-1) 
      {
	 matched = false; /* Indicates whether the next node is matched. */
	    
	 for(i=1; i<=currNode->condBranchCount; i++)
	 {
	    if(strcmp(currNode->nextNode[i]->className, mappingTable.mappingNode[chIdx].className) == 0)
	    {		
	       if(PRINTLOG)
		  fprintf(logFP," *%s* ", mappingTable.mappingNode[chIdx].className);

	       currNode = currNode->nextNode[i]; 
	       matched = true;
	       break;
	    }   
	 }
      }
	      
      if(matched && ((wordIdx-1) != strlen(inputWord)-1))
      {
	 /* Check for word limit end detection has to be put here.           
	  * Look for further matching nodes since best matching rule is used. 
	  */
	 continue; 
      }
      else
      {
	 int contextCnt = 0;

	 /* There is a rule which matches to the current position but not anymore. 
	  */
	 if(currNode->isLast) 
	 {		
	    if(PRINTLOG)
	       fprintf(logFP," :: APPLIED ::\n");

	    PerformAction(currNode, mappingTable, inputWord, startIdx, wordIdx);

	    /* The rule matches till the current position and hence the starting index is reset */
		  
	    if(strcmp(ruleModifier,"Y")==0)
	    {
	       if((wordIdx-startIdx)>2) 
	       { 
		  --wordIdx;
		  flag=1;
	       }		  		 
	    }

	    --wordIdx;
    
	    startIdx = wordIdx; 
		 		  
	    /* Break if this is the last character in the word. 
	     */
	    if(startIdx == strlen(inputWord)-1) 
	    {
	       ++startIdx;
		    
	       break;
	    }

	    /* This resets the flag, however unnecessary since already false.  
	     */
	    matched = false; 
	 }
	 else /* No matching rule found, simple replacement to be used */
	 {  
	    if(PRINTLOG)
	       fprintf(logFP," :: NOT APPLIED ::\n");

	    if(flag == 0)
	    {
	       /* Changed chIdx to startIDx */
	       StorePhoneString(ReturnInteger(inputWord[startIdx]), \
				mappingTable.mappingNode[ReturnInteger(inputWord[startIdx])].mappedStr); 
	    }
	    ++startIdx;

	    wordIdx = startIdx; /* word index is aligned with the start index          */
	    matched = false;    /* Actually reduntant since already 'matched' is false */
	    flag = 0;
	 }
      }	
	       
   }

   /* This is executed to input any of the remaining mappings which were half matched in the previous loop */

   while(startIdx <= strlen(inputWord)-1 && flag==0) 
   {	   
      chIdx = ReturnInteger(inputWord[startIdx]); /* Gets the integer equivalent of the current character */

      StorePhoneString(chIdx, mappingTable.mappingNode[ReturnInteger(inputWord[startIdx])].mappedStr); /* chIdx used here is new */
      ++startIdx;	     
   }

   if(PRINTLOG)
      fprintf(logFP,"\n-----------------------------------------------------\n");
	
}

/* Stores the character integer equivalent and the mapped phone string */

void RuleBase::StorePhoneString(int idx, char *mappedStr)
{
   phoneticEq.characterIdx[phoneticEq.fragmentCount] = idx;
   strcpy(phoneticEq.phoneStr[phoneticEq.fragmentCount], mappedStr);
   phoneticEq.fragmentCount++;
}

/* Returns the number equivalent of a character */
      
int RuleBase::ReturnInteger(char ch)
{
   unsigned int idx; 

   idx = (unsigned int) ch;
   idx &= 0x000000ff;	 
	 
   return(idx);
}

/* This method implements the action part of a rule */
      
void RuleBase::PerformAction(RuleNode *actionNode, MappingTable mappingTable, char *inputWord, int startIdx, int wordIdx)
{      
   if((flag == 1) && (actionNode->nextNode[0] != NULL))  
      actionNode = actionNode->nextNode[0]->nextNode[0];

   while(actionNode!= NULL)
   {
      if(actionNode->action == 'R')
	 R(actionNode, mappingTable, inputWord, startIdx, wordIdx);

      if(actionNode->action == 'K')
	 K(actionNode, mappingTable, inputWord, startIdx, wordIdx);
	    
      if(actionNode->action == 'A')
	 A(actionNode, mappingTable, inputWord, startIdx, wordIdx);

      if(actionNode->action == 'I')
	 I(actionNode, mappingTable, inputWord, startIdx, wordIdx);

      /* Other types of actions have to be checked here */
	    
      actionNode = actionNode->nextNode[0]; /* Go further down the action list */
   }
}
      
/* Replacement Function */ 
      
void RuleBase::R(RuleNode *actionNode, MappingTable mappingTable, char *inputWord, int startIdx, int wordIdx)
{
   int workingIdx, chIdx;

   workingIdx = startIdx + actionNode->position - 1; /* Creates the working index as per the rule */
   chIdx = ReturnInteger(inputWord[workingIdx]);     /* Get the index of the current chcracter    */

   StorePhoneString(chIdx, actionNode->targetStr);
}

/* Keeps whatever is matched */ 

void RuleBase::K(RuleNode *actionNode, MappingTable mappingTable, char *inputWord, int startIdx, int wordIdx)
{
   int workingIdx, chIdx;

   workingIdx = startIdx + actionNode->position - 1; /* Creates the working index as per the rule */
   chIdx = ReturnInteger(inputWord[workingIdx]);     /* Get the index of the current chcracter    */

   StorePhoneString(chIdx, mappingTable.mappingNode[chIdx].mappedStr); /* This does the work of retaining */
}

/* Append additional material to the matched part */ 

void RuleBase::A(RuleNode *actionNode, MappingTable mappingTable, char *inputWord, int startIdx, int wordIdx)
{
   int workingIdx, chIdx;
   char tmpMappedStr[MAXSTRLENGTH];

   workingIdx = startIdx + actionNode->position - 1; /* Creates the working index as per the rule */
   chIdx = ReturnInteger(inputWord[workingIdx]);     /* Get the index of the current chcracter    */

   strcpy(tmpMappedStr, "");
   strcpy(tmpMappedStr, mappingTable.mappingNode[chIdx].mappedStr);
   strcat(tmpMappedStr, actionNode->targetStr);

   StorePhoneString(chIdx, tmpMappedStr); /* This does the work of retaining */
}

/* Inserts a New Phone */ 

void RuleBase::I(RuleNode *actionNode, MappingTable mappingTable, char *inputWord, int startIdx, int wordIdx)
{
   int workingIdx, chIdx;

   workingIdx = startIdx + actionNode->position - 1; /* Creates the working index as per the rule */
   chIdx = ReturnInteger(inputWord[workingIdx]);     /* Get the index of the current chcracter    */

   StorePhoneString(chIdx, actionNode->targetStr); /* This does the work of inserting */
}

/* Reads the file containing rules and builds data structures. 
 */
void RuleBase::CreateRuleBase(char *ruleBaseFileName)
{
   int  i, idx;
   int  conditionRegion;    /* Used to track the action part of a rule        */
   /* 0: Condition, 1: Action Part                   */
   bool newRule;            /* Indiates the starting of a new rule            */
   bool ruleMatch;          /* Indiactes whether initial part of the current  */
   /* rule matches some already existing rule        */
   RuleNode *currNode;      /* Points to the node within the rule structure   */
   /* during the rule structure building             */
   RuleHeadNode tmpHeadNode;
   bool skipRule;

   int  ruleCount = 0;

   ifstream ruleFP(ruleBaseFileName);
   string   className;

   conditionRegion = 1;
   newRule = true;

   skipRule = false;

   while(ruleFP >> className) 
   {          
      if(className == "{") /* Indicates the starting of action of a rule */
      {
	 conditionRegion = 0;

	 currNode->isLast = 1; /* Indicates that this can be the last testing condition */

	 continue;
      }
      
      if(className == "}") /* Indicates the ending of action part */
      {
	 conditionRegion = 1;
	 newRule = true;
	 skipRule = false;

	 /** After every rule, set the lastNode ptr to the beginning of the rule(s) within 
	  *  that head so that the new rule can be matched from the beginning
	  */
	 ruleHead[idx].lastNode = ruleHead[idx].head; 
	              
	 ++ruleCount;
                                      
	 continue;
      }

      if(skipRule)
	 continue;

      if(newRule)
      {
	 newRule = false; /* Reset the new rule indicator */
	 ruleMatch = false;

	 for(i=0; i<ruleHead.size(); i++)
	 {		 
	    if(ruleHead[i].head != NULL)
	    {	      
	       if(strcmp(ruleHead[i].head->className, className.c_str()) == 0)
	       {
		  /* If the intial part matches then look down under that head */		
		 
		  ruleMatch = true;  
		  break;
	       }
	    }
	 }

	 if(ruleMatch) /* Partial match found within the existing rules */
	 {
	    currNode = ruleHead[i].head;
	    ruleHead[i].ruleCount++;
	    idx = i;
	    continue;
	 }
	 else /* Current rule doesn't match any of the existing rules */
	 {  
	    tmpHeadNode.ruleCount = 1;

	    /* Assuming that a new rule will start only with a condition node */

	    tmpHeadNode.head = AllocateRuleNode(conditionRegion, 0, className.c_str()); 

	    tmpHeadNode.lastNode = tmpHeadNode.head;

	    ruleHead.push_back(tmpHeadNode);
	    idx = ruleHead.size() - 1;     /* Vector index starts from 0    */

	    currNode = ruleHead[idx].head; /* Corrected after bug detection */

	    /* cout << "Kattu\n"; */
	    /* To know more about Kattu, mail ptpartha@india.hp.com :)) */
	 }
      }
      else
      {
	 /* Depending on action region, the next node is selected. 
	  */ 
	 currNode = ruleHead[idx].lastNode->nextNode[conditionRegion]; 
     
	 if(currNode == NULL) /* No node exists after this node with the given type */
	 {	  	   
	    ruleHead[idx].lastNode->nextNode[conditionRegion] = AllocateRuleNode(conditionRegion, 0, className.c_str());
	    currNode = ruleHead[idx].lastNode->nextNode[conditionRegion];

	    if(conditionRegion == 0) /* For action nodes */
	    {
	       /* Split the action string and store into action node; Format -> A:1:str */
	       /* Error checking not done here */

	       sscanf(className.c_str(),"%c:%d:%s", &(currNode->action), &(currNode->position), currNode->targetStr);
	    }
	    else
	       ruleHead[idx].lastNode->condBranchCount++;	   

	    ruleHead[idx].lastNode = currNode; /* Point the last node to the newly alloted node */	    	    
	 }
	 else /* Some part is already matched, now we have to check whether among the available child 
	       *  nodes, any match occurs
	       */
	 {
	    /* Backtrack to the parent node since we have to search among the available children nodes */

	    currNode = ruleHead[idx].lastNode; 

	    if(conditionRegion == 0)
	    {
	       cout << "\n\t** Error: Two rules with same condition part not permitted **";
	       cout << "\n\t**                 Retaining the first rule                **\n\n";

	       skipRule = true; /* Skip the current rule */
	    }
	    else
	    {
	       if(currNode->condBranchCount == MAXBRANCHCOUNT)
	       {
		  cout << "\n\t** Maximum rule branching reached **";
		  cout << "\n\t**     Skipping Current Rule      **\n\n";

		  skipRule = true; /* Skip the current rule since maximum branching has been reached */
	       }

	       bool alreadyPresent = false;

	       /* Index starts from 1 since 0 is reserved for action nodes and condition node index
		* starts from 1                                                                     
		*/
	       for(i=1; i<=currNode->condBranchCount; i++)
	       {
		  if(strcmp(ruleHead[idx].lastNode->nextNode[i]->className, className.c_str()) == 0)
		  {
		     alreadyPresent = true;

		     ruleHead[idx].lastNode = currNode->nextNode[i];

		     currNode = ruleHead[idx].lastNode; 
			   
		     break;
		  }
	       }
		  
	       if(alreadyPresent == false)
	       {
		  /* currNode->nextNode[condBranchCount
		   * Add the node with the new className 
		   * This condition will be used if more than two rules have some initial part same
		   * e.g. A B C D { 1 2 }
		   *      A B E   { 3 4 }
		   *      A B C M { 5 6 }
		   */
		  currNode->condBranchCount++;

		  currNode->nextNode[currNode->condBranchCount] = AllocateRuleNode(conditionRegion, 0, className.c_str());
			
		  ruleHead[idx].lastNode = currNode->nextNode[currNode->condBranchCount];			

		  currNode = ruleHead[idx].lastNode;
	       }
	    }
	 }
      }	
   } 

   ruleFP.close(); /* Close the rule file pointer. */
}

void RuleBase::PrintRuleBase()
{
   int i;

   for(i=0; i<ruleHead.size(); i++)
   {
     cout << ">> " << ruleHead[i].head->className << " ";
      Print(ruleHead[i].head);
	    
   }
}

void RuleBase::Print(RuleNode *nodePtr)
{
   int i;

   if(nodePtr == NULL)
   {
      cout << "\n";
      return;
   }

   // cout << nodePtr->className << " " << nodePtr->isLast << " " << nodePtr->condBranchCount << " "; 
   cout << nodePtr->className << " "; 
	 
   for(i=0; i<=nodePtr->condBranchCount; i++)
   {
      Print(nodePtr->nextNode[i]);
   }
}

RuleBase::RuleNode * RuleBase::AllocateRuleNode(int type, int isLast, const char str[MAXSTRLENGTH])
{
   RuleNode * tmpNode;
   int        i;

   tmpNode = (RuleNode *) malloc(sizeof(RuleNode));

   tmpNode->type = type;
   tmpNode->isLast = isLast;

   if(type == 0)
      strcpy(tmpNode->className, str); /* Example of conversion from string to array. */
   else
      strcpy(tmpNode->className, str);  /* Just set it to NULL */

   tmpNode->condBranchCount = 0;

   tmpNode->nextNode[0] = NULL;
   tmpNode->nextNode[1] = NULL;

   /* Initialize all next node pointers to NULL */
   for(i=0; i<=MAXBRANCHCOUNT; i++)
     tmpNode->nextNode[i] = NULL;

   return(tmpNode);
}

