/************************************************************************************/
/* Copyright (c) 2003, Outside Echo Ltd. & Hewlett-Packard.                         */
/* All rights reserved.                                                             */
/*                                                                                  */
/* Redistribution and use in source and binary forms, with or without modification, */
/* are permitted provided that the following conditions are met:                    */
/*                                                                                  */
/*    * Redistributions of source code must retain the above copyright notice, this */
/*      list of conditions and the following disclaimer.                            */
/*    * Redistributions in binary form must reproduce the above copyright notice,   */
/*      this list of conditions and the following disclaimer in the documentation   */
/*      and/or other materials provided with the distribution.                      */
/*    * Neither the name of Outside Echo Ltd. & Hewlett-Packard                     */
/*      nor the names of its contributors may be used to endorse or promote products*/
/*      derived from this software without specific prior written permission.       */
/*                                                                                  */
/*                                                                                  */
/*   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS NAMELY     */
/*   OUTSIDE ECHO LTD AND HEWLETT-PACKARD "AS IS" AND ANY                           */
/*   EXPRESS  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/*   WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/*   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE    */
/*   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     */
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR     */
/*   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER     */
/*   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  */
/*   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  DAMAGE.          */
/*                                                                                  */
/************************************************************************************/
/* This file contains the classes for rule structure creation                       */
/*                                                                                  */
/*                                                                                  */
/* AUTHOR: Partha Pratim Talukdar, Hewlett-Packard Labs (India)                     */
/* Email: (ptpartha@india.hp.com)                                                   */
/* CREATED:  Oct 10, 2003                                                           */
/* MODIFIED: Jan 7, 2004                                                            */
/*                                                                                  */
/************************************************************************************/

#ifndef __RULEBASE_H_
#define __RULEBASE_H_

#include "headers.h"
#include "Mapping.h"

// #include <vector>
#include "Vector.h"

using namespace std;

/* Instance of this class srored the rule base.
 */

class RuleBase
{
  struct RuleNode
  {
    int type;                                    /* Specifies a condition or an action node         */
    int isLast;                                  /* Indicates whether the current node is the last  */
    /* condition node                                  */
    char className[MAXCLASSLEN];                 /* Stores the classname to be checked incase this  */
    /* is a condition node                             */
    char action;                                 /* Specifies the action to be executed             */

    int  position;                               /* Position at which the rule needs to be applied  */
    char targetStr[MAXSTRLENGTH];                /* Stores the string needed for implementing       */
    /* the target action                               */

    struct RuleNode *nextNode[MAXBRANCHCOUNT+1]; /* Points to the next condition/action node        */
    int    condBranchCount;                      /* Specifies the number of branch spinoffs from current node */
  };

  typedef struct RuleNode RuleNode;

  struct RuleHeadNode
  {
    RuleNode *head;                          /* Points to the first element in the rule chain     */
    RuleNode *lastNode;                      /* Points to the last node in the rule chain         */
    int      ruleCount;                      /* Total number of rules below this rule head        */
  };

  typedef struct RuleHeadNode RuleHeadNode;

  FILE *logFP;                                  /* This pointer logs the activities of the Rulebase matcher */

  typedef struct
  {
    unsigned int characterIdx[MAXWORDLEN];  /* Stores the integer of each character so that later */
    /* lookup is easier                                   */
    char phoneStr[MAXWORDLEN][MAXSTRLENGTH];/* Stores the phonetic strings                        */
    int  fragmentCount;                     /* Stores the total number of phonetic segments       */

  } PhoneticEq;                                 /* Strores the phonetic equivalent of th word         */

 public:

  Vector<RuleHeadNode> ruleHead;

  PhoneticEq phoneticEq;  /* Stores the phoneme strings */
  int flag;

 public:

  /* Constructor */
  RuleBase();

  /* This method does the function of matching the rules against the current input */
  /* and performs necessary action                                                 */
  int Match(char *inputWord, MappingTable mappingTable, char *ruleModifier);

  /* Stores the character integer equivalent and the mapped phone string */
  void StorePhoneString(int idx, char *mappedStr);

  /* Returns the number equivalent of a character */
  int ReturnInteger(char ch);

  /* This method implements the action part of a rule */
  void PerformAction(RuleNode *actionNode, MappingTable mappingTable, char *inputWord, int startIdx, int wordIdx);

  /* Replacement Function */
  void R(RuleNode *actionNode, MappingTable mappingTable, char *inputWord, int startIdx, int wordIdx);

  /* Keeps whatever is matched */
  void K(RuleNode *actionNode, MappingTable mappingTable, char *inputWord, int startIdx, int wordIdx);

  /* Append additional material to the matched part */
  void A(RuleNode *actionNode, MappingTable mappingTable, char *inputWord, int startIdx, int wordIdx);

  /* Inserts a New Phone */
  void I(RuleNode *actionNode, MappingTable mappingTable, char *inputWord, int startIdx, int wordIdx);

  /* Reads the file containing rules and builds data structures. */
  void CreateRuleBase(char *ruleBaseFileName);

  void PrintRuleBase();

  void Print(RuleNode * nodePtr);

 private:

  RuleNode * AllocateRuleNode(int type, int isLast, const char str[MAXSTRLENGTH]);
};


#endif
