
#ifndef __VECTOR_H_
#define __VECTOR_H_

#include "headers.h"
#include "error.h"

template<class T>
class Vector
{
 private:
  
  int vectorSize;
  T   vectorArray[MAXWORDLEN];

 public:

  Vector();

  /* The reference was put as a bug fix: took time to locate :) */
  T& Vector<T>::operator[] (int index)
  {
     return(vectorArray[index]);
  }
    
  void push_back(T );

  void insert(int, T);

  T    at(int );

  int  size();
  
  void clear();
};

template<class T>
Vector<T>::Vector()
{
  vectorSize = 0;
}

template<class T>
void Vector<T>::push_back(const T element)
{
  if(vectorSize >= MAXWORDLEN)
  {
    // Error::Print("Vector Size Exceeded .. Returning");
     printf("\nVector Size Exceeded .. Returning\n\n");
     return;
  }

  vectorArray[vectorSize] = element;
  vectorSize++;
}

/* This method should be used if pointers are stored in the container. */
/* Using push_back causes a problem when pointers are stored in the    */
/* container without allocating memory. The problem arises when the    */
/* memory contents pointed to by 'element' changes after call to this  */
/* Hence, here first memory is allocated and then the contents are     */
/* copied.                                                             */
/*                                                                     */
template<class T>
void Vector<T>::insert(int size, T element)
{
  if(vectorSize >= MAXWORDLEN)
  {
    printf("\nVector Size Exceeded .. Returning\n\n");
    return;
  }

  Vector::vectorArray[Vector::vectorSize] = (T) malloc(size + 1);

  memcpy(vectorArray[vectorSize], element, size);
 
  Vector::vectorArray[Vector::vectorSize][size] = 0;

  Vector::vectorSize++;
}

template<class T>
T Vector<T>::at(int index)
{
   return(vectorArray[index]);
}

template<class T>
int Vector<T>::size()
{
   return(vectorSize);
}

template<class T>
void Vector<T>::clear()
{
   vectorSize = 0;
}

#endif
