/************************************************************************************/
/* Copyright (c) 2003, Outside Echo Ltd. & Hewlett-Packard.                         */
/* All rights reserved.                                                             */
/*                                                                                  */
/* Redistribution and use in source and binary forms, with or without modification, */
/* are permitted provided that the following conditions are met:                    */
/*                                                                                  */
/*    * Redistributions of source code must retain the above copyright notice, this */
/*      list of conditions and the following disclaimer.                            */
/*    * Redistributions in binary form must reproduce the above copyright notice,   */
/*      this list of conditions and the following disclaimer in the documentation   */
/*      and/or other materials provided with the distribution.                      */
/*    * Neither the name of Outside Echo Ltd. & Hewlett-Packard                     */
/*      nor the names of its contributors may be used to endorse or promote products*/
/*      derived from this software without specific prior written permission.       */
/*                                                                                  */
/*                                                                                  */
/*   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS NAMELY     */
/*   OUTSIDE ECHO LTD AND HEWLETT-PACKARD "AS IS" AND ANY                           */
/*   EXPRESS  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/*   WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/*   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE    */
/*   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     */
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR     */
/*   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER     */
/*   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  */
/*   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  DAMAGE.          */
/*                                                                                  */
/************************************************************************************/
/* Class for Hindi Grapheme to Phoneme Conversion                                   */
/*                                                                                  */
/*                                                                                  */
/* AUTHOR: Partha Pratim Talukdar, Hewlett-Packard Labs (India)                     */
/* Email: (ptpartha@india.hp.com)                                                   */
/* CREATED:  Aug 14, 2003                                                           */
/* MODIFIED: Jan 7, 2004                                                            */
/* MODIFIED: Feb, 2005 Roger Tucker - added dummy POS function                      */
/*                                                                                  */
/************************************************************************************/

#ifndef __G2P_H__
#define __G2P_H__

#include "headers.h"
#include "error.h"
#include "RuleBase.h"
#include "Mapping.h"
#include "lexhash.h"

#include "festival.h"

#include "Vector.h"

using namespace std;

/* Instance of this class performs the core of Hindi Grapheme to
 * Phoneme Conversion.
 */
class G2P
{
      ifstream    iFP;
      FILE      * oFP;
      RuleBase ruleBase;          /* This object is used to match the rules for the current input */
      char inputWord[MAXWORDLEN]; /* Stores the input word                                        */
      char inpWord[MAXWORDLEN];
      char genWord[MAXWORDLEN];
      MappingTable mappingTable;  /* This is the mapping table, letter to sound mapping */

      Vector<char *> phoneStack;  /* Holds the phone strings                            */

      HashTable lexHash;

   public:

      /* Original Constructor */
      G2P(char *, char *, char *, char *, char *);

      /* Constructor for Integration with Festival */
      G2P(char *, char *, char *);

      /* This method does the conversion on a word by word basis. */
      void Run(char *, char *);

      /* Version Used for Integration with Festival */
      void Run(char *, char *, char const *, Vector<char *>&);

      /* Prints the results to the outputfile */
      void Print();

      /* This method takes care of Tamil Schwa insertion          */
      /* Added by deepa on 30th Oct for the "a" and AB BC problem */

      void TamilSchwaInsertion();

      int ReturnInteger(char ch);

      /* This method handles Hindi schwa deletion. */
      void HindiSchwaDeletion();

      /* Schwa Deletion module integrated with Festival */
      void HindiSchwaDeletion(Vector<char *>&);
};

/* HP Labs India Hindi Text G2P */
LISP HPLHindiG2P(LISP );

/* Dummy test starting point */
LISP Start(LISP, LISP);

/* This routine is needed for integrating G2P with Festival */
LISP InitiateG2P(char const *);

/* Dummy POS */
LISP HPLDummyPOS(LISP );

#endif
