/************************************************************************************/
/* Copyright (c) 2003, Outside Echo Ltd. & Hewlett-Packard.                         */
/* All rights reserved.                                                             */  
/*                                                                                  */
/* Redistribution and use in source and binary forms, with or without modification, */
/* are permitted provided that the following conditions are met:                    */
/*                                                                                  */
/*    * Redistributions of source code must retain the above copyright notice, this */
/*      list of conditions and the following disclaimer.                            */
/*    * Redistributions in binary form must reproduce the above copyright notice,   */
/*      this list of conditions and the following disclaimer in the documentation   */
/*      and/or other materials provided with the distribution.                      */
/*    * Neither the name of Outside Echo Ltd. & Hewlett-Packard                     */
/*      nor the names of its contributors may be used to endorse or promote products*/ 
/*      derived from this software without specific prior written permission.       */
/*                                                                                  */ 
/*                                                                                  */
/*   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS NAMELY     */
/*   OUTSIDE ECHO LTD AND HEWLETT-PACKARD "AS IS" AND ANY                           */
/*   EXPRESS  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/*   WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/*   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE    */
/*   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     */
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR     */
/*   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER     */
/*   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  */
/*   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  DAMAGE.          */
/*                                                                                  */
/************************************************************************************/
/* Class for Hindi Grapheme to Phoneme Conversion                                   */
/*                                                                                  */
/*                                                                                  */
/* AUTHOR: Partha Pratim Talukdar, Hewlett-Packard Labs (India)                     */
/* Email: (ptpartha@india.hp.com)                                                   */
/* CREATED:  Aug 14, 2003                                                           */
/* MODIFIED: Jan 7, 2004                                                            */
/* MODIFIED: Feb, 2005 Roger Tucker - added dummy POS function                      */
/*                                                                                  */
/************************************************************************************/

// #include <vector>

#include "Vector.h"

#include <string>
#include "festival.h"
#include "g2p.h"
#include "stdlib.h"

#include "../Lexicon/lex_aux.cc"

#define FEST_DIR /home/sameer/softwares/temp/festival_latest/

using namespace std;

EST_Item * AddSegment(EST_Utterance *, EST_String &);
static EST_Item * AddSyllable(EST_Utterance *, int);
EST_Item * AddSegment(EST_Utterance *u, const char * strPtr);

/* HP Labs India Phonetiser */
LISP HPLHindiG2P(LISP utt)
{
   EST_Item * wordIter;
   EST_Item * tokenIter;

   /**
    * This list is filled with the outout of the G2P and 
    * passed on to the syllabification module.
    */
   EST_StrList  segmentList;
   LISP         syllableList, p, s;
   EST_Item   * syl;
   EST_Item   * segmentName;

   Vector<char *> phoneticOutputString;
   int            i;

   EST_Utterance * u = get_c_utt(utt);

      char hindi_rules[512], hindi_mapping[512], hindi_lexicon[512] ;

 /* if FEST_DIR  not set - QUIT */

      if ((char *)getenv("FEST_DIR"))
	{
      sprintf(hindi_rules, "%s/src/modules/hpl_hin_g2p/Hindi/hindi_rules", getenv("FEST_DIR"));

      sprintf(hindi_mapping, "%s/src/modules/hpl_hin_g2p/Hindi/hindi_mapping", getenv("FEST_DIR"));

      sprintf(hindi_lexicon, "%s/src/modules/hpl_hin_g2p/Hindi/new_hindi_lexicon", getenv("FEST_DIR"));
	}    
else   
 {
    printf("FEST_DIR environment variable not set.\n Please set the same to your festival installation path \n");
    exit(0);
  } 

 G2P g2p (hindi_rules, hindi_mapping, hindi_lexicon) ;
	

   u->create_relation("Segment");
   u->create_relation("Syllable");

   /* Iterate over tokens and change tokens into words */       
   for(wordIter = u->relation("Word")->first(); wordIter != 0; wordIter = wordIter->next_leaf())
   {
      phoneticOutputString.clear();	
      
      /* Phonetise Current Word */
      g2p.Run("H", "N", wordIter->name().str(), phoneticOutputString);
   
      /* Clear the segment list */
      segmentList.clear();

      /** 
       * For each word, iterate through the output phonetic string 
       * and associate the segments (phones) with the word.
       */
      for(i=0; i<phoneticOutputString.size(); i++)
      {
	 if(strcmp(phoneticOutputString[i], "#") == 0)
	    continue;
      
	 /**
	  * Wastage of resources here. Efficient implementation is needed here.
	  * Just a temporary work around.
	  */
	 EST_String newSegmentName(phoneticOutputString[i]);
      
	 segmentList.append(newSegmentName);	
      }

      /* Syllabify the segments(phones) */
      syllableList = lex_syllabify_phstress(siod_strlist_to_list(segmentList));	
   
      for (s=syllableList; s != NIL; s=cdr(s))
      {
	 syl = AddSyllable(u, get_c_int(car(cdr(car(s)))));	    

	 // append_daughter(newWord, "SylStructure", syl);
	 append_daughter(wordIter, "SylStructure", syl);

	 for (p=car(car(s)); p != NIL; p=cdr(p))
	 {
	    segmentName = AddSegment(u, siod_sprint(car(p)));	       	
	    append_daughter(syl, "SylStructure", segmentName);
	 }
      }		
	 
      printf("\n~~");
   }       

   return utt;
}

/* Constructor Defined for Integration with Festival */

// G2P::G2P(char * ruleFileName, char * mappingFileName, char * lexiconFileName, char * outputFileName)
G2P::G2P(char * ruleFileName, char * mappingFileName, char * lexiconFileName)
{
   oFP = stdout;

   /*
   if((oFP = fopen(outputFileName, "w")) == NULL)
   {
      cout << "\nError opening temporary phonetic output file. Aborting Program.\n\n";
      exit(1);
   }
   */

   /* Load the character to phoneme mapping into memory. */
   mappingTable.LoadMappingFile(mappingFileName);  

   /* Build the data structure which stores the rule base. */
   ruleBase.CreateRuleBase(ruleFileName);

   /* Set the output file pointer. */
   lexHash.SetOpFilePtr(lexiconFileName, oFP);     

   /* Load the lexicon into memory */
   lexHash.InsertLex();
}

/* Version Used for Integration with Festival.
 * This method does the conversion on a word by word basis. 
 * Phonetizes just one word.
 */
void G2P::Run(char *languageCode, char *ruleModifier, char const * word, Vector<char *>& phoneticOutput)
{
   int  ruleResult;
   int  wordCnt, tmpOffset,compCnt; 
   char Word[MAXWORDLEN];
   // std::string tmpStr;
   // char *tmpSentence; /* Array used to read in the sentence */
   int  i, l, senCount;
   bool isCompound;


   /********************************************************/
   /* Workaround for avoiding the constant pointer problem.*/
   strcpy(Word, word);
   /********************************************************/

   wordCnt = 0;
   compCnt = 0;
   senCount = 0;
   	  
   isCompound = false;
   // tmpOffset += strlen(Word) + 1;  /* Something more robust is needed here */

   if(strstr(Word, " ") != NULL)
   {
      //continue;
      cout << "NULL String: Returning\n";
      return;
   }

   if(strlen(Word) >= MAXWORDLEN) 
   {
      cout << "Skipping Word :: Longer than allowed length\n"; 
      // continue;	     
      return;
   }
	   
   Word[strlen(Word)] = 0;

   /* Print the initial hash */
   fprintf(oFP, "#");

   /* Look up whether the current word is present in the phonetic 
    * compound lexicon.
    */
   isCompound = lexHash.LookUp(Word, phoneticOutput);

   if(isCompound) 
   {
      ++compCnt;
      fprintf(oFP, " #");
      return;
   }
	       
   i = 0;
   l = 0;

   /* Changed on 28-10-03 for enclosing the input words with '#' */

   l = strlen(Word);
   inpWord[0] = '#';

   /* For two # and the ending NULL character. 
    */
   if(l < MAXWORDLEN - 3) 
   {
      for(i=0; i<l; i++)
      {
	 inpWord[i+1] = Word[i];
      }

      inpWord[i+1] = '#';
      inpWord[i+2] = 0;
   }
   else
   {
      cout << "Current word is too long: Skipping this word" << endl;
      // continue;
      return;
   }
	   
   strcpy(inputWord, inpWord);

   ++wordCnt;

   if(strcmp(languageCode, "T") == 0)     /* For any unknown language */
      TamilSchwaInsertion();

   ruleResult = ruleBase.Match(inputWord, mappingTable, ruleModifier);

   if(strcmp(languageCode, "H") == 0)          /* Schwa Deletion: A phenomenon in Hindi */
      HindiSchwaDeletion(phoneticOutput);                    
   else if(strcmp(languageCode, "T") == 0)     /* For Tamil.                            */
      Print();
   else if(strcmp(languageCode, "U") == 0)     /* For any unknown language              */	 
      Print();

   strcpy(inputWord,"");  /* Clear off the string */
   strcpy(inpWord,"");    /* Clear off the string */
   strcpy(Word,"");       /* Clear off the string */
   phoneStack.clear();    /* Clear off the stack  */

   // if(tmpOffset<strlen(tmpSentence)-1) 
   fprintf(oFP, " #");

   // strcpy(tmpSentence,"");
	
   // tmpSentence[0] = 0;

   fprintf(oFP," \n");

   // printf("\n\nTotal %d words processed\n", wordCnt);
   // printf("\nTotal %d compounds processed\n\n", compCnt);
}

 
/* This method handles Hindi schwa deletion. */
/* Written to integrated with Festival       */
void G2P::HindiSchwaDeletion(Vector<char *>& phoneticOutput)
{
   int i, tmpIdx;
   bool   halantOrEnd, isWordEnd; /* This flag indicates whether current context is halanth or word end */

   /* Set since the first iteration starts from the end.
    */
   halantOrEnd = true; 
   
   for(i=ruleBase.phoneticEq.fragmentCount-1; i>=0; i--)
   {
      if(i == ruleBase.phoneticEq.fragmentCount-1) 
	 isWordEnd = true;
      else 
	 isWordEnd = false;

      tmpIdx = ruleBase.phoneticEq.characterIdx[i];

      if(mappingTable.mappingNode[tmpIdx].type == 'H') /* Check for halanth */
      {
	 halantOrEnd = true;	    
	 continue;
      }

      if(!halantOrEnd && mappingTable.mappingNode[tmpIdx].type == 'C')
      { 
	 if(i==0)
	 {
	    phoneStack.push_back("a");
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);
	    continue;
	 }
	 /* Added by deepa on oct 16th for "prakAr" problem*/
	 else if(i>=2 && ((strcmp(mappingTable.mappingNode[tmpIdx].className,"RA")==0) && \
			  (mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'H') && \
			  (mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-2]].type == 'C')))
	 {
	    phoneStack.push_back("a");
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-2]);
	    i=i-2;
	    continue;
	 } /* End of code for "prakAr" */
	 else if(mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'V')
	 {
	    if(i>=2 && (mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-2]].type == 'C'))
	    {
	       phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);   /* C */
	       phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-1]); /* V */
	       phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-2]); /* C */
		    
	       /* Decrement the index, the third decrement is taken care of by the for loop. 
		*/
	       i = i - 2; 
	       continue;
	    }
	    else
	    {
	       /* Added by deepa on Oct 14th diff from dhvani VC.
		*/
	       phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);   /* C */
	       phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-1]); /* V */
	       i =i - 1;
	       continue;
	    }
	 }
	 else if(mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'C')
	 {
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);       /* C */
	    phoneStack.push_back("a");                                   /* a */
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-1]);     /* C */

	    i = i - 1; /* Decrement the index, the second decrement is taken care of by the for loop */
	    continue;
	 }
	 else if(mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'H')
	 {
	    /* Order changed here for better results C A C changed to C C A */
	    phoneStack.push_back("a");                                 /* a */ 
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);     /* C */
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-2]);   /* C */

	    i = i - 2; /* Decrement the index, the third decrement is taken care of by the for loop */
	    continue;
	 }
      }
      else if((!halantOrEnd  || (i==ruleBase.phoneticEq.fragmentCount-1)) && mappingTable.mappingNode[tmpIdx].type == 'V')
      {
	 halantOrEnd=false;

	 if((i==0) || (mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'V'))
	 {
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);		 
	    continue;
	 }
	 else if(mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'C')
	 {
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);
   
	    /* Added by deepa on Oct 15 for "pandit" problem */
	    if(mappingTable.mappingNode[tmpIdx].type=='S' && \
	       (mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'C'))
	    {
	       phoneStack.push_back("a");
	    } 

	    /* Added by deepa on Oct 15 for "gaye" problem */
	    if((strcmp(mappingTable.mappingNode[tmpIdx].className,"VS_1")==0) && \
	       (mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'C'))
	    {
	       phoneStack.push_back("a");
	    } 
	
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-1]); 
 
	    i = i - 1; /* Decrement the index, the second decrement is taken care of by the for loop */
	       
	    continue;
	 }
	 else if(mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'H')
	 {
	    /* It is assumed here that the character preceding ahalant will be a consoant
	     * Error checking possible here                                               
	     */
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);   
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-2]);
		  
	    /* Decrement the index, the second decrement is taken care of by the for loop */
	    i = i - 2; 

	    continue;
	 }
      }
      else if(halantOrEnd && (mappingTable.mappingNode[tmpIdx].type == 'C')) /* This case maps to 'Ch' in the Dhvani Algo */
      {
	 halantOrEnd = false; /* Resets the flag */

	 if(i==0)
	 {
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);
	    continue;
	 }
	 else if(mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'S') 
	 {
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-1]);

	    i = i - 1;

	    continue;
	 }
	 else if(mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'V')
	 {
	    if(i>=2 && (mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-2]].type == 'C'))
	    {
	       phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);   /* C */
	       phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-1]); /* V */
	       phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-2]); /* C */
	      
	       /* Decrement the index, the third decrement is taken care of by the for loop */
	       i = i - 2; 

	       continue;
	    }
	    else
	    { 
	       phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);   /* C */
	       phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-1]); /* V */

	       /* Decrement the index, the third decrement is taken care of by the for loop */
	       i = i - 1; 

	       continue;
	    }
	 }
	 else if(mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'C')
	 {
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);   /* C */
	    phoneStack.push_back("a");
	    phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i-1]); /* C */

	    /* Decrement the index, the second decrement is taken care of by the for loop */
	    i = i - 1; 

	    continue;
	 }
	 else if(mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-1]].type == 'H')
	 {
	    /* It is assumed here that the character preceding a halant will be a consoant.
	     * Error checking possible here.
	     */

	    /* Two lookaheads since at i-1 position there is a halant */

	    if(i>=2 && mappingTable.mappingNode[ruleBase.phoneticEq.characterIdx[i-2]].type == 'C') 
	    {
	       /* Added on 28th oct for consonant cluster at the end.
		*/
	       /* Commented again on 2-4-2004 */
	       //if(isWordEnd)  
	       //phoneStack.push_back("*a*");		      
		   
	       phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);     /* C */		    
	       continue;
	    }
	    else
	    {
	       phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);   /* C */
	       continue;
	    }
	 }	    
      }

      /* Added for "hain" problem */

      if(i != 0)
	 phoneStack.push_back(ruleBase.phoneticEq.phoneStr[i]);

      halantOrEnd=false;
   }

   /* Prints the phones in the correct order. 
    */
   for(i=phoneStack.size()-1; i>=0; i--) 
   {
      fprintf(oFP, " %s", phoneStack[i]);
      
      phoneticOutput.push_back(phoneStack[i]);
   }
}

EST_Item * AddSegment(EST_Utterance *u, EST_String &t)
{
   /* Add a word item to the end of Segment relation */
   EST_Item *item = u->relation("Segment")->append();

   item->set_name(t);    
	
   return item;
}

EST_Item * AddSegment(EST_Utterance *u, const char * strPtr)
{
   EST_String t(strPtr);

   /* Add a word item to the end of Segment relation */
   EST_Item *item = u->relation("Segment")->append();
   
   item->set_name(t);    
	
   return item;
}

static EST_Item * AddSyllable(EST_Utterance *u, int stress)
{
   EST_Item *item = u->relation("Syllable")->append();
    
   item->set_name("syl");
   item->set("stress",stress);

   return item;
}
      

/* Interface between G2P and Festival */
/*
LISP InitiateG2P(char const * word)
{
   LISP tmpLispPtr;    

   G2P g2p("/home/partha/project/TTS-INTSINT/festival/src/modules/hpl_hin_g2p/Hindi/hindi_rules", "/home/partha/project/TTS-INTSINT/festival/src/modules/hpl_hin_g2p/Hindi/hindi_mapping", "/home/partha/project/TTS-INTSINT/festival/src/modules/hpl_hin_g2p/Hindi/new_hindi_lexicon", "/home/partha/__phonetic_tmp");   

   g2p.Run("H","N", word);

   printf("HPL Indian Hindi G2P\n");

   return(tmpLispPtr);
}
*/

/* Dummy POS */
LISP HPLDummyPOS(LISP utt)
{
   EST_Item * wordIter;
   EST_Utterance * u = get_c_utt(utt);

   /* Iterate over words */       
   for(wordIter = u->relation("Word")->first(); wordIter != 0; wordIter = wordIter->next_leaf())
   {
        wordIter->set("pos","nil");
   }       

   return utt;
}
