/************************************************************************************/
/* Copyright (c) 2003, Outside Echo Ltd. & Hewlett-Packard.                         */
/* All rights reserved.                                                             */  
/*                                                                                  */
/* Redistribution and use in source and binary forms, with or without modification, */
/* are permitted provided that the following conditions are met:                    */
/*                                                                                  */
/*    * Redistributions of source code must retain the above copyright notice, this */
/*      list of conditions and the following disclaimer.                            */
/*    * Redistributions in binary form must reproduce the above copyright notice,   */
/*      this list of conditions and the following disclaimer in the documentation   */
/*      and/or other materials provided with the distribution.                      */
/*    * Neither the name of Outside Echo Ltd. & Hewlett-Packard                     */
/*      nor the names of its contributors may be used to endorse or promote products*/ 
/*      derived from this software without specific prior written permission.       */
/*                                                                                  */ 
/*                                                                                  */
/*   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS NAMELY     */
/*   OUTSIDE ECHO LTD AND HEWLETT-PACKARD "AS IS" AND ANY                           */
/*   EXPRESS  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/*   WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/*   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE    */
/*   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     */
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR     */
/*   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER     */
/*   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  */
/*   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  DAMAGE.          */
/*                                                                                  */
/************************************************************************************/
/* This file contains the impementation of Hash.                                    */
/*                                                                                  */
/*                                                                                  */
/* AUTHOR: Deepa S.R., Hewlett-Packard Labs (India)                                 */
/* Email: deepasr@india.hp.com                                                      */
/* CREATED:  Oct 20, 2003                                                           */
/* MODIFIED: Jan 7, 2004                                                            */
/*                                                                                  */
/************************************************************************************/

#include "lexhash.h"
#include "error.h"
 
/* List to store all the suspision compound words */

void HashTable::SetOpFilePtr(char *lexFileName, FILE *fp2)
{
   if((flex = fopen(lexFileName, "r")) == NULL)
      Error::PrintAndExit("Lexicon File Could Not be Opened");
	 
   fout = fp2;   
}

HashTable::HashTable()
{
   HashTable::Initialise();
}

/* Opens the lexicon and inserts the compounds and 
 * corresponding phones
 */
void HashTable::InsertLex()
{
   char compound[MAXWORDLEN], phoneticRep[MAXWORDLEN];

   while(fscanf(flex, "%s %[^\n]\n", compound, phoneticRep) != EOF)
   {
      Insert(compound, phoneticRep);
   }

   // printf("Lexicon Loading ... Over\n");
}

/* Insertion of the compound word      */
void HashTable::Insert(char *compound,char *phoneticRep)
{
   struct list *L,*p;
   unsigned int cell;

   cell = findIndex(compound);
   L = H->head[cell];
   p = (List) malloc(sizeof(struct list));
   strcpy(p->compound,compound);
   strcpy(p->phoneticRep,phoneticRep);
   p->next = L->next;
   L->next = p;
} 

/* Searching the word.
 * One or more words can hash to the same cell.
 */
bool HashTable::LookUp(char *compound)
{	
   struct list *L,*prev;
   unsigned int cell;
   int flag=0;

   cell = findIndex(compound);
   L = H->head[cell];

   while(L != NULL)
   {	
      /* If the word is there print it in output file
       * and delete it from the list
       */

      if(strcmp(L->compound, compound)==0)
      {
	 fprintf(fout," %s", L->phoneticRep); 
	 return(true);
      }
   
      L = L->next;
   }

   return(false);
}     

HashTable::Hash  HashTable::Initialise()
{
   int i;
   
   H = (Hash) malloc(sizeof(struct hash));
   H->tableSize = TSIZE;
   H->head = (List *) malloc(sizeof(struct list)*H->tableSize);

   for(i=0; i<H->tableSize; i++)
   {
      H->head[i] = (List) malloc(sizeof(struct list));
      H->head[i]->next = NULL;
   }

   return H;
}

/* Hash function to get the index by hashing 
 * on the second word.
 */
unsigned int HashTable::findIndex(char *key)
{
   unsigned int hashval=0;

   while(*key!='\0') hashval=(hashval<<5) + *key++;
   return hashval % TSIZE;
}
