/************************************************************************************/
/* Copyright (c) 2003, Outside Echo Ltd. & Hewlett-Packard.                         */
/* All rights reserved.                                                             */  
/*                                                                                  */
/* Redistribution and use in source and binary forms, with or without modification, */
/* are permitted provided that the following conditions are met:                    */
/*                                                                                  */
/*    * Redistributions of source code must retain the above copyright notice, this */
/*      list of conditions and the following disclaimer.                            */
/*    * Redistributions in binary form must reproduce the above copyright notice,   */
/*      this list of conditions and the following disclaimer in the documentation   */
/*      and/or other materials provided with the distribution.                      */
/*    * Neither the name of Outside Echo Ltd. & Hewlett-Packard                     */
/*      nor the names of its contributors may be used to endorse or promote products*/ 
/*      derived from this software without specific prior written permission.       */
/*                                                                                  */ 
/*                                                                                  */
/*   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS NAMELY     */
/*   OUTSIDE ECHO LTD AND HEWLETT-PACKARD "AS IS" AND ANY                           */
/*   EXPRESS  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/*   WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/*   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE    */
/*   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     */
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR     */
/*   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER     */
/*   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  */
/*   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  DAMAGE.          */
/*                                                                                  */
/************************************************************************************/
/* This file contains the impementation of Hash.                                    */
/*                                                                                  */
/*                                                                                  */
/* AUTHOR: Deepa S.R., Hewlett-Packard Labs (India)                                 */
/* Email: deepasr@india.hp.com                                                      */
/* CREATED:  Oct 20, 2003                                                           */
/* MODIFIED: Jan 7, 2004                                                            */
/*                                                                                  */
/************************************************************************************/

#ifndef __LEXHASH_H_
#define __LEXHASH_H_

#include "headers.h"

#include "Vector.h"

/* List to store all the suspision compound words */

class HashTable
{
      typedef struct list
      {
	    char compound[MAXWORDLEN];
	    char phoneticRep[MAXWORDLEN];
	    struct list *next;
      } * List;
      
      /* HashTable definition */

      typedef struct hash
      {
	    int tableSize;
	    List *head;
	
      } * Hash;
          
      FILE *fout; 
      FILE *flex;
      Hash H;
      
      /* Initialising the hash table by allocating memory */
      
   public:
 
      void SetOpFilePtr(char *, FILE *);     

      HashTable();     

      /* Opens the lexicon and inserts the compounds and 
       * corresponding phones
       */
      void InsertLex();      

      /* Insertion of the compound word      */
      void Insert(char *, char *);      

      /* Searching the word.
       * One or more words can hash to the same cell.
       */
      bool LookUp(char *compound);      

      bool LookUp(char *compound, Vector<char *>&);      
      
   private:

      Hash  Initialise();      

      /* Hash function to get the index by hashing 
       * on the second word.
       */
      unsigned int findIndex(char *);     
};	

#endif
