
/************************************************************************************/
/* Copyright (c) 2003, Outside Echo Ltd. & Hewlett-Packard.                         */
/* All rights reserved.                                                             */  
/*                                                                                  */
/* Redistribution and use in source and binary forms, with or without modification, */
/* are permitted provided that the following conditions are met:                    */
/*                                                                                  */
/*    * Redistributions of source code must retain the above copyright notice, this */
/*      list of conditions and the following disclaimer.                            */
/*    * Redistributions in binary form must reproduce the above copyright notice,   */
/*      this list of conditions and the following disclaimer in the documentation   */
/*      and/or other materials provided with the distribution.                      */
/*    * Neither the name of Outside Echo Ltd. & Hewlett-Packard                     */
/*      nor the names of its contributors may be used to endorse or promote products*/ 
/*      derived from this software without specific prior written permission.       */
/*                                                                                  */ 
/*                                                                                  */
/*   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS NAMELY     */
/*   OUTSIDE ECHO LTD AND HEWLETT-PACKARD "AS IS" AND ANY                           */
/*   EXPRESS  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/*   WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/*   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE    */
/*   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     */
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR     */
/*   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER     */
/*   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  */
/*   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  DAMAGE.          */
/*                                                                                  */
/************************************************************************************/
/*                                                                                  */
/*                                                                                  */
/* AUTHOR: Panchapagesan. K., Hewlett-Packard Labs (India)                                 */
/* Email: sameer.badaskar@hp.com                                                      */
/* CREATED:  April 2, 2004                                                           */
/* MODIFIED: June 17, 2004                                                            */
/*                                                                                  */
/************************************************************************************/

#include "Token_to_Words.h"

char ** NumberWords;
char ** Ordinals;
char ** DenominationsModel1;
char ** DenominationsModel2;
char ** Month_Names;

void Init_Data_Structures()
{
  /*
    Initializes the various arrays with words taken from an input file
   */
	FILE *fp;
	int i,j,len;
	char *temp,c;
        char hindi_numbers_iscii[512];

 /* if FEST_DIR  not set - QUIT */

      if ((char *)getenv("FEST_DIR"))
	{
 sprintf(hindi_numbers_iscii, "%s/src/modules/hpl_normalizer/Numbers.isc", getenv("FEST_DIR"));
	}
   else   
 {
    printf("FEST_DIR environment variable not set.Please set the same to your festival installation path. \n");
    exit(0);
  } 

	fp = fopen(hindi_numbers_iscii,"r");
	if(fp == NULL)
	{
	  	printf("\"Numbers.isc\" File Not Found\n");
		exit(0);
	}
	temp = (char *)malloc(20*sizeof(char));
	NumberWords = (char **)malloc(100*sizeof(char *));
	Ordinals = (char **)malloc(5*sizeof(char *));
	DenominationsModel1 = (char **)malloc(4*sizeof(char *));
	DenominationsModel2 = (char **)malloc(4*sizeof(char *));
	Month_Names = (char **)malloc(12*sizeof(char *));
	for(i=0;i<100;i++)
	{
	  //		printf("Reading NumberWords %d...\n", i);
		len = 0;
		while((c=fgetc(fp)) != '\n' && c!= EOF) temp[len++] = c;
		NumberWords[i] = (char *)malloc((len+1)*sizeof(char));
		for(j=0;j<len;j++) NumberWords[i][j] = temp[j];
		NumberWords[i][len] = '\0';
		//		printf("Done\n");
	}
	for(i=0;i<4;i++)
	{
	  //		printf("Reading DenominationsModel1 %d\n", i);
		len = 0;
		while((c=fgetc(fp)) != '\n' && c!= EOF) temp[len++] = c;
		DenominationsModel1[i] = (char *)malloc((len+1)*sizeof(char));
		for(j=0;j<len;j++) DenominationsModel1[i][j] = temp[j];
		DenominationsModel1[i][len] = '\0';
	}
	for(i=0;i<4;i++)
	{
	  //		printf("Reading DenominationModel2 %d\n", i);
		len = 0;
		while((c=fgetc(fp)) != '\n' && c!= EOF) temp[len++] = c;
		DenominationsModel2[i] = (char *)malloc((len+1)*sizeof(char));
		for(j=0;j<len;j++) DenominationsModel2[i][j] = temp[j];
		DenominationsModel2[i][len] = '\0';
	}
	for(i=0;i<12;i++)
	{
	  //		printf("Reading DenominationModel2 %d\n", i);
		len = 0;
		while((c=fgetc(fp)) != '\n' && c!= EOF) temp[len++] = c;
		Month_Names[i] = (char *)malloc((len+1)*sizeof(char));
		for(j=0;j<len;j++) Month_Names[i][j] = temp[j];
		Month_Names[i][len] = '\0';
	}
	fclose(fp);
	return;
}


unsigned int StringtoInt(const char *str, int Iscii, int begin, int end )
{
  /*
    Converts a given string of digits into the number the digits sequence represents
    str: the given input string of digit
    Iscii: Indicates whether the given input string is in hindi(Iscii == 1) or not
    begin: indicates the position on the given input string where the conversion must begin
    end: indicates the position on the given input string where the conversion must end
   */
	unsigned int val=0;
	int i;
	//	printf("Entering StringtoInt\n");
	if(Iscii)
	{
		for(i=begin;i<=end;i++)
		{
		  //	printf("%d", ((unsigned char)str[i]));
			if( str[i] != ',') val = val*10 + (unsigned char)str[i] - 241;
		}
	}
	else
	{
		for(i=begin;i<=end;i++)
		{
			if(str[i] != ',') val = val*10 + (unsigned char)str[i] - 48;
		}
	}	
	//	printf("\nExiting StringtoInt\n");
	return val;
}

int Get_Number_Array(const char *NumberString, int Iscii,int *No_of_Digits, int Digit_Array_Size, unsigned int **Result_Array)
{
  /*
    Takes in an Input String and extracts the Numbers of various sizes as specified in No_Of_Digits onto Result Array.
    NumberString: Given Input string of digits
    Iscii: Indicates whether the given input string is in hindi(Iscii == 1) or not
    No_of_Digits: Gives the size pattern in number of digits, to be extracted from the given string.
    Digit_Array_Size: indicates the size of the No_Of_Digits Array.
    Result Array: Addressof the pointer to an integer array, in to which the numbers based on the size pattern will be extracted into
    Return value: The Numbers in *Result_Array and the actual count of numbers that were extracted.
   */
  int begin,end,NumberArrayLength=0;
  int i;
  //  printf("Entering GetNumberArray\n");
  (*Result_Array) = (unsigned int *)malloc(Digit_Array_Size*sizeof(unsigned int));
  end = strlen(NumberString);
	for(i=0;i<Digit_Array_Size; i++)
	  {
	    //    printf("i = %d\n",i);
	    if((begin=end-No_of_Digits[i])>0) 
	      {
		(*Result_Array)[i]=StringtoInt(NumberString,Iscii,begin,end-1);
		//printf("%d\n", (*Result_Array)[i]);
		NumberArrayLength++;
		end=begin;
	      }
	    else
	      {
		(*Result_Array)[i]=StringtoInt(NumberString,Iscii,0,end-1);
		//		printf("%d\n", (*Result_Array)[i]);
		NumberArrayLength++;
		break;
	      }
	  }
	//  printf("Exiting  GetNumberArray\n");
  return NumberArrayLength;
}

int isNum(unsigned char val, int Iscii)
{
  /*
    Checks to see if a given input character is a number or not
    val: Given input character
    Iscii: Indicates whether the given input string is in hindi(Iscii == 1) or not
    Returns: 1 If the given character is a number, 0 otherwise
   */
  switch(Iscii)
    {
    case 1:
      {
	if(val>=241 && val <=250) return 1;
	else return 0;
      }
    case 0:
      {
	if(val>='0' && val <='9') return 1;
	else return 0;
      }
    }
}

char *SubString(const char *str, int begin, int end)
{
  /*
    Returns the sub-string of the given input string
    str: The given Input String
    begin: Indicates the position where the substring required begins
    end: Indicates the position on the given string where the required substring ends
   */
  char *string;
  int i,j;
  //  printf("Entering SubString\n");
  string=(char *)malloc((end - begin + 2)*sizeof(char));
  for(i=begin,j=0;i<=end;i++,j++) string[j]=str[i];
  string[j]='\0';
  return string;
}

char *RemoveCommas(const char *str)
{
  /*
    Converts the given input string with non-Digit characters interspread, 
    to one with only Digits in it and returns the converted string.
   */
	char *str2;
	int i,j;
	str2=(char *)malloc((strlen(str)+1)*sizeof(char));
	j=0;
	for(i=0;i<strlen(str);i++) 
	{
		if(str[i] != ',')
		{
			str2[j]=str[i];
			j++;
		}
	}
	str2[j]='\0';
	return str2;
}

int Convert_NUM_to_Words(const char *NumberString, int Iscii, int Model, EST_StrList& wordList)
{
  /*
    Converts a given input Number in String format to its Corresponding words
    NumberString: The given input string of Numbers
    Iscii: Indicates whether the given input string is in hindi(Iscii == 1) or not
    Model: Indicates the Number Model to be used
    Model 1: Indian Style Model - Crores, Lakhs, Thousands, Hundreds, Tens
    Model 2: American Style Model - Billions, Millions, Thousands, Hundreds and Tens
    Model 3: The model used to express YEAR in words
   */
  unsigned int *NumberArray;
  int NumberArrayLength;
  int *No_Of_Digits,DigitArraySize;
  int NumberSize;
  char *LowerOrderDigits;
  int i;
  NumberSize = strlen(NumberString);
  //  printf("Entering convertNumtoWord with Model %d\n", Model);
  //  Added by Sameer on 23/11/2004
  if(atoi(NumberString) == 0)
  {
	wordList.append(EST_String(NumberWords[0]));
	return 1;
  }

  switch(Model)
    {
    case 1:  {
      DigitArraySize = 3;
      No_Of_Digits = (int *)malloc(DigitArraySize*sizeof(int));
      No_Of_Digits[0]=3;
      No_Of_Digits[1]=2;
      No_Of_Digits[2]=2;
      if(NumberSize>7)
	{
	  Convert_NUM_to_Words(SubString(NumberString,0,NumberSize - 1 - 7),Iscii,Model, wordList);
	  wordList.append(EST_String(DenominationsModel1[3]));
	  NumberArrayLength = Get_Number_Array(SubString(NumberString,NumberSize - 7,NumberSize -1),Iscii,No_Of_Digits, DigitArraySize, &NumberArray);
	}
      else NumberArrayLength = Get_Number_Array(NumberString,Iscii, No_Of_Digits, DigitArraySize, &NumberArray);

      for(i=NumberArrayLength - 1; i>0 ; i--)
	{
	  if(Number_to_Words(NumberArray[i], wordList))
	    wordList.append(EST_String(DenominationsModel1[i]));
	}
      Number_to_Words(NumberArray[0], wordList);
      break;
    }
    case 2: {
      DigitArraySize = 3;
      No_Of_Digits = (int *)malloc(DigitArraySize*sizeof(int));
      No_Of_Digits[0]=3;
      No_Of_Digits[1]=3;
      No_Of_Digits[2]=3;
      if(strlen(NumberString)>9)
	{
	  Convert_NUM_to_Words(SubString(NumberString,0,NumberSize - 1 - 9),Iscii,Model, wordList);
	  wordList.append(EST_String(DenominationsModel2[3]));
	  NumberArrayLength = Get_Number_Array(SubString(NumberString,NumberSize - 9,NumberSize -1),Iscii,No_Of_Digits, DigitArraySize, &NumberArray);
	}
      else  NumberArrayLength = Get_Number_Array(NumberString,Iscii, No_Of_Digits, DigitArraySize, &NumberArray);
     
      for(i=NumberArrayLength - 1; i>0 ; i--)
	{
	  if(Number_to_Words(NumberArray[i], wordList))
	    wordList.append(EST_String(DenominationsModel2[i]));
	}
      Number_to_Words(NumberArray[0],wordList);
      break;
    }
    case 3: {
      if(atoi(NumberString)>=2000) Convert_NUM_to_Words(NumberString,Iscii,1, wordList);
      else
	{
	  DigitArraySize = 2;
	  No_Of_Digits = (int *)malloc(DigitArraySize*sizeof(int));
	  No_Of_Digits[0]=2;
	  No_Of_Digits[1]=2;
	  NumberArrayLength = Get_Number_Array(NumberString,Iscii,No_Of_Digits, DigitArraySize, &NumberArray);
	  for(i=NumberArrayLength - 1; i>0 ; i--)
	    {
	      if(Number_to_Words(NumberArray[i],wordList))
		wordList.append(EST_String(DenominationsModel1[0]));
	    }
	  Number_to_Words(NumberArray[0],wordList);

	}
      break;
    }
    }
  //  printf("Exiting convertNumtoWord\n");
}


void Convert_NDATE_to_Words(const char *Date, int Iscii, int Format, EST_StrList& wordList)
{
  /*
    Converts the given Date to corresponding words
    Date: The input string containing the Date
    Iscii: Indicates whether the given input string is in hindi(Iscii == 1) or not
    Format: Indicates the Format of the given Date
    Format 1: DMY
    Format 2: MDY
    Format 3: YMD
    Format 4: DM
    Format 5: MD
    Format 6: MY
    Format 7: YM
   */
  char *Day,*Month,*Year;
  int begin,end,length;
  //  printf("\nEntering Convert_NDATE_to_Words\n\n");
  length=strlen(Date);
  switch(Format)
    {
    case 1:
      {
	//	printf("DATE FORMAT: %d\n", Format);
	begin=0;
	end=begin;
	while(isNum(Date[end],Iscii)) end++;
	Day = SubString(Date,begin,end-1);
	//	printf("DAY: %s\n", Day);
	begin=end;
	while(!isNum(Date[begin],Iscii)) begin++;
	end=begin;
	while(isNum(Date[end],Iscii)) end++;
	Month = SubString(Date,begin,end-1);
	//	printf("MONTH: %s\n", Month);
	begin=end;
	while(!isNum(Date[begin],Iscii)) begin++;
	end=begin;
	while(end!=length && isNum(Date[end],Iscii)) end++;
	Year = SubString(Date,begin,end-1);
	//	printf("YEAR: %s\n", Year);
	Number_to_Words(StringtoInt(Day,Iscii,0,strlen(Day)-1), wordList);
	wordList.append(EST_String(Month_Names[StringtoInt(Month,Iscii,0,strlen(Month)-1)-1]));
	Convert_NUM_to_Words(Year,Iscii,3, wordList);
	break;
      }
    case 2:
      {
	begin=0;
	end=begin;
	while(isNum(Date[end],Iscii)) end++;
	Month = SubString(Date,begin,end-1);
	begin=end;
	while(!isNum(Date[begin],Iscii)) begin++;
	end=begin;
	while(isNum(Date[end],Iscii)) end++;
	Day = SubString(Date,begin,end-1);
	begin=end;
	while(!isNum(Date[begin],Iscii)) begin++;
	end=begin;
	while(end!=length && isNum(Date[end],Iscii)) end++;
	Year = SubString(Date,begin,end-1);
	Number_to_Words(StringtoInt(Day,Iscii,0,strlen(Day)-1), wordList);
	wordList.append(EST_String(Month_Names[StringtoInt(Month,Iscii,0,strlen(Month)-1)-1]));
	Convert_NUM_to_Words(Year,Iscii,3, wordList);
	break;
      }
    case 3:
      {
	begin=0;
	end=begin;
	while(isNum(Date[end],Iscii)) end++;
	Year = SubString(Date,begin,end-1);
	begin=end;
	while(!isNum(Date[begin],Iscii)) begin++;
	end=begin;
	while(isNum(Date[end],Iscii)) end++;
	Month = SubString(Date,begin,end-1);
	begin=end;
	while(!isNum(Date[begin],Iscii)) begin++;
	end=begin;
	while(end!=length && isNum(Date[end],Iscii)) end++;
	Day = SubString(Date,begin,end-1);
	Number_to_Words(StringtoInt(Day,Iscii,0,strlen(Day)-1), wordList);
	wordList.append(EST_String(Month_Names[StringtoInt(Month,Iscii,0,strlen(Month)-1)-1]));
	Convert_NUM_to_Words(Year,Iscii,3, wordList);
	break;
      }
    case 4:
      {
	begin=0;
	end=begin;
	while(isNum(Date[end],Iscii)) end++;
	Day = SubString(Date,begin,end-1);
	begin=end;
	while(!isNum(Date[begin],Iscii)) begin++;
	end=begin;
	while(end!=length && isNum(Date[end],Iscii)) end++;
	Month = SubString(Date,begin,end-1);
	Number_to_Words(StringtoInt(Day,Iscii,0,strlen(Day)-1), wordList);
	wordList.append(EST_String(Month_Names[StringtoInt(Month,Iscii,0,strlen(Month)-1)-1]));
	break;
      }
    case 5:
      {
	begin=0;
	end=begin;
	while(isNum(Date[end],Iscii)) end++;
	Month = SubString(Date,begin,end-1);
	begin=end;
	while(!isNum(Date[begin],Iscii)) begin++;
	end=begin;
	while(end!=length && isNum(Date[end],Iscii)) end++;
	Day = SubString(Date,begin,end-1);
	Number_to_Words(StringtoInt(Day,Iscii,0,strlen(Day)-1), wordList);
	wordList.append(EST_String(Month_Names[StringtoInt(Month,Iscii,0,strlen(Month)-1)-1]));
	break;
      }
    case 6:
      {
	//	printf("DATE Format: %d, Iscii: %d\n", Format,Iscii);
	begin=0;
	end=begin;
	while(isNum(Date[end],Iscii)) end++;
	Month = SubString(Date,begin,end-1);
	begin=end;
	//	printf("MONTH: %s\n", Month);
	while(!isNum(Date[begin],Iscii)) begin++;
	end=begin;
	while(end!=length && isNum(Date[end],Iscii)) end++;
	Year = SubString(Date,begin,end-1);
	//	printf("YEAR: %s\n", Year);
	wordList.append(EST_String(Month_Names[StringtoInt(Month,Iscii,0,strlen(Month)-1)-1]));
	Convert_NUM_to_Words(Year,Iscii,3,wordList);
	break;
      }
    case 7:
      {
	begin=0;
	end=begin;
	while(isNum(Date[end],Iscii)) end++;
	Year = SubString(Date,begin,end-1);
	begin=end;
	while(!isNum(Date[begin],Iscii)) begin++;
	end=begin;
	while(end!=length && isNum(Date[end],Iscii)) end++;
	Month = SubString(Date,begin,end-1);
	wordList.append(EST_String(Month_Names[StringtoInt(Month,Iscii,0,strlen(Month)-1)-1]));
	Convert_NUM_to_Words(Year,Iscii,3, wordList);
	break;
      }

    }
  //  printf("\nExiting Convert_NDATE_to_Words\n\n");
  return;
}


void Convert_NFLOAT_to_Words(const char *Float, int Iscii,int Format, EST_StrList& wordList)
{
  int begin, end;
  char *Num1,*Num2;
  int length;
  length = strlen(Float);
  begin=0;
	end=begin;
	while(isNum(Float[end],Iscii)) end++;
	Num1 = SubString(Float,begin,end-1);
	begin=end;
	while(!isNum(Float[begin],Iscii)) begin++;
	end=begin;
	while(end!=length && isNum(Float[end],Iscii)) end++;
	Num2 = SubString(Float,begin,end-1);
	Number_to_Words(StringtoInt(Num1,Iscii,0,strlen(Num1)-1), wordList);
	//	wordList.append(EST_String(Month_Names[StringtoInt(Month,Iscii,0,strlen(Month)-1)-1]));
	Convert_NLITERAL_to_Words(Num2,Iscii,0, strlen(Num2), wordList);
	return;
}

void Convert_NLITERAL_to_Words(const char *Literal, int Iscii,int begin, int end, EST_StrList& wordList)
{
  /*
    Converts the given input String of Digits to a sequence of words treating each digit as a separate entity
    Literal: The input string of digits
    Iscii: Indicates whether the given input string is in hindi(Iscii == 1) or not
    begin: Indicates the start position on the input string from where the conversion is to begin
    end: Indicates where on the string, the conversion must end
   */
  int i;
  if(Iscii)
    {
      for(i=begin;i<=end;i++)
	{
	  wordList.append(EST_String(NumberWords[(unsigned int)Literal[i] - 241]));
	}
    }
  else
    {
      for(i=begin;i<=end;i++)
	{
	  wordList.append(EST_String(NumberWords[(unsigned int)Literal[i] - '0']));
	  // Added by sameer Insert commas
	  wordList.append(EST_String(","));
	}
    }

  return;
}

void Convert_NTEL_to_Words(const char *Tel, int Iscii,int Format, EST_StrList& wordList)
{
  /*
  Convert Telephone Number Strings to corresponding words
  Tel is the input Telephone number
  Iscii == 1 indicates that the string is in Hindi
  Format {1-6} : Indicates the format of the given Telephone Number
  Format 1: Only the Number without Std or Country Code
  Format 2: A Pure String of digits may or may not contain the STD and Country codes
  Format 3 & 4: STD Code followed by the actual telephone number
  Format 5 & 6: Country code followed by STD code followed by the actual telephone number
  */
  int begin,end;
  switch(Format)
    {
    case 1:
      {
	begin=0;
	end=begin;
	while(isNum(Tel[end],Iscii)) end++;
	Convert_NLITERAL_to_Words(Tel,Iscii,begin,end-1, wordList);
	begin=end;
	while(!isNum(Tel[begin],Iscii)) begin++;
	end=begin;
	while(isNum(Tel[end],Iscii)) end++;
	Convert_NLITERAL_to_Words(Tel,Iscii,begin,end-1, wordList);
	break;
      }
    case 2:
      {
	Convert_NLITERAL_to_Words(Tel,Iscii,0,strlen(Tel)-1, wordList);
	break;
      }
    case 3:
    case 4:
      {
	begin=0;
	while(!isNum(Tel[begin],Iscii)) begin++;
	end=begin;
	while(isNum(Tel[end],Iscii)) end++;
	Convert_NLITERAL_to_Words(Tel,Iscii,begin,end-1, wordList);
	while(end!=strlen(Tel))
	  {
	    begin=end;
	    while(!isNum(Tel[begin],Iscii)) begin++;
	    end=begin;
	    while(isNum(Tel[end],Iscii)) end++;
	    Convert_NLITERAL_to_Words(Tel,Iscii,begin,end-1,wordList);
	  }
	break;
      }
    case 5:
    case 6:
      {
	begin=0;
	while(!isNum(Tel[begin],Iscii)) begin++;
	end=begin;
	while(isNum(Tel[end],Iscii)) end++;
	Convert_NLITERAL_to_Words(Tel,Iscii,begin,end-1, wordList);
	begin=end;
	while(!isNum(Tel[begin],Iscii)) begin++;
	end=begin;
	while(isNum(Tel[end],Iscii)) end++;
	Convert_NLITERAL_to_Words(Tel,Iscii,begin,end-1, wordList);
	while(end!=strlen(Tel))
	  {
	    begin=end;
	    while(!isNum(Tel[begin],Iscii)) begin++;
	    end=begin;
	    while(isNum(Tel[end],Iscii)) end++;
	    Convert_NLITERAL_to_Words(Tel,Iscii,begin,end-1, wordList);
	  }
	break;
      }
    }
  return;
}

int Number_to_Words(unsigned int val, EST_StrList& wordList)
{
  //Used for printing basic numbers < 1000 ,  as whatever be the model, these strings will have the same Output format
  int returnval=0;
  int tempval;
  //  printf("Entering NumbertoWord with value %d\n", val);
  if(tempval=(val/100))
    {
      wordList.append(EST_String(NumberWords[tempval]));
      wordList.append(EST_String(DenominationsModel1[0]));
      returnval=1;
    }
  if(tempval=(val%100))
    {
      wordList.append(EST_String( NumberWords[tempval]));
      returnval=1;
    }
  //  printf("Exiting NumbertoWord\n");
  return returnval;
}

void Token_to_Words(const char *token, int isIscii, int tokenType, int tokenFormat, EST_StrList& wordList)
{
  switch(tokenType)
    {
    case NUM:
      {
	Convert_NUM_to_Words(RemoveCommas(token), isIscii, tokenFormat, wordList);
	break;
      }
    case NYEAR:
      {
	Convert_NUM_to_Words(token, isIscii, tokenFormat, wordList);
	break;
      }
    case NDATE:
      {
	Convert_NDATE_to_Words(token,isIscii,tokenFormat, wordList);
	break;
      }
    case NTEL:
      {
	Convert_NTEL_to_Words(token, isIscii,tokenFormat,wordList);
	break;
      }
    case NTIME:
      {
	Convert_NFLOAT_to_Words(token, isIscii,tokenFormat, wordList); // temporary soln
	break;
      }
    case NFLOAT:
      {
	Convert_NFLOAT_to_Words(token, isIscii,tokenFormat, wordList);//temporary soln
	break;
      }
    default:
      {
	wordList.append(EST_String(token));
	break;
      }	
    }
  return;
}
