
/************************************************************************************/
/* Copyright (c) 2003, Outside Echo Ltd. & Hewlett-Packard.                         */
/* All rights reserved.                                                             */  
/*                                                                                  */
/* Redistribution and use in source and binary forms, with or without modification, */
/* are permitted provided that the following conditions are met:                    */
/*                                                                                  */
/*    * Redistributions of source code must retain the above copyright notice, this */
/*      list of conditions and the following disclaimer.                            */
/*    * Redistributions in binary form must reproduce the above copyright notice,   */
/*      this list of conditions and the following disclaimer in the documentation   */
/*      and/or other materials provided with the distribution.                      */
/*    * Neither the name of Outside Echo Ltd. & Hewlett-Packard                     */
/*      nor the names of its contributors may be used to endorse or promote products*/ 
/*      derived from this software without specific prior written permission.       */
/*                                                                                  */ 
/*                                                                                  */
/*   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS NAMELY     */
/*   OUTSIDE ECHO LTD AND HEWLETT-PACKARD "AS IS" AND ANY                           */
/*   EXPRESS  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/*   WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/*   DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE    */
/*   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     */
/*   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR     */
/*   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER     */
/*   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  */
/*   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  DAMAGE.          */
/*                                                                                  */
/************************************************************************************/
/*                                                                                  */
/*                                                                                  */
/* AUTHOR: Panchapagesan. K., Hewlett-Packard Labs (India)                                 */
/* Email: sameer.badaskar@hp.com                                                      */
/* CREATED:  April 2, 2004                                                           */
/* MODIFIED: June 17, 2004                                                            */
/*                                                                                  */
/************************************************************************************/


#include <iostream.h>
#include "Tokenizer.h"
#include "headers.h"
#include "festival.h"
#include "text.h"
#include <fstream.h>
#include <stdlib.h>
#include <iconv.h>




#include "../hpl_hin_g2p/g2p.h"
#include "../hpl_hin_g2p/Vector.h"

// #include "../Lexicon/lex_aux.cc"

/*** For Tokenization and Normalization ***/
#include "lex.h"
#include "lex.cc"

#include "Token_to_Words.h"

#define BUFSIZE 4096

EST_Item * AddWord(EST_Utterance *, EST_Item *);
EST_Item * AddWord(EST_Utterance *, EST_String &);

void Process_NSW(const char *, EST_Item *, const char *, EST_StrList& );
void Process_Token(EST_Item *, const char *, EST_StrList&);
bool isNSW(const char *token);


/* HP Labs India Hindi Text Tokenizer */

LISP HPLHindiTokenizer(LISP utt)
{
   EST_Utterance * u = get_c_utt(utt);
   EST_String      inpFileName;   
   EST_String      tokenTextFeature("text"); /* Token feature which stores the text of ambiguous tokens */

   /* For Text Normalization Routines */
   YY_BUFFER_STATE string_buffer;
   EST_StrList     tokenList;
   int             count;
      std::string  text;
      //EST_String text;

   ifstream  iFP;

   EST_Litem * tokenListIter;   /* Token list iterator */
   EST_Item  * tokenItem;
  
   *cdebug << "Text module\n";

   /* text stores the name of the input file */
      inpFileName = get_c_string(utt_iform(*u));



   /* Create token relation */
   u->create_relation("Token");

   iFP.open(inpFileName);

   /* Initializes Data Structures for Text Normalization */
   Init_Data_Structures();
   
      while(getline(iFP,text))
      {      
      /* Lex method for tokenization */	    
         string_buffer = yy_scan_bytes(text.c_str(), strlen(text.c_str()));
   //      string_buffer = yy_scan_bytes(text, strlen(text));
      yylex(TOKEN, -1, tokenList);                         

      /* Iterate over each token and insert them into the utterance structure */
      for(tokenListIter = tokenList.head(); tokenListIter != 0; tokenListIter = next(tokenListIter))
      {
	 EST_Token tok(tokenList.item(tokenListIter));      
	 tokenItem = add_token(u, tok);

	 if(isNSW(tokenList.item(tokenListIter).str()))
	 {	
	    /* Access the text of the current token which is stored in the next node              */
	    /* The token and its text are stored in subsequent tokens for programming convenience */
	    tokenListIter = next(tokenListIter);
	 
	    printf(">>>> %s -- %s\n", tokenList(tokenListIter).str(), tokenList.item(tokenListIter).str());
	  
	    tokenItem->set(tokenTextFeature, tokenList.item(tokenListIter).str());
	 }
      }
      }  // end of while
   
   return utt;
}

/* HP Labs India Hindi Normalizer */

LISP HPLHindiNormalizer(LISP utt)
{     
   EST_Item * tokenIter;
   EST_Item * newWord;  
   EST_Litem * wordListIter;    /* Word list iterator */
   EST_Relation * sylStructure; /* SylStructure Relation */
   EST_StrList wordList;        /* Word list which contains the words once a token is disambiguated */

   /* For Text Normalization Routines */
   YY_BUFFER_STATE string_buffer;

   EST_Litem * tokenListIter;   /* Token list iterator                 */
   EST_Item  * tokenItem;       /* token item pointer of type EST_Item */   
   
   EST_String  tokenTextFeature("text"); /* Token feature which stores the text of ambiguous tokens */      

   EST_Utterance * u = get_c_utt(utt);

   *cdebug << "Normalization module\n";

   /* Initializes Data Structures for Text Normalization */
   Init_Data_Structures();
   	
   /* Create Relations */
   u->create_relation("Word");
   sylStructure = u->create_relation("SylStructure");
                
   /* Iterate over tokens and change tokens into words */       
   for(tokenIter = u->relation("Token")->first(); tokenIter != 0; tokenIter = tokenIter->next_leaf()) 
   {
      printf("\n>> %s\n", tokenIter->name().str());		

      /* Clear the word list */ 
      wordList.clear();

      /* Process each token */
      if(tokenIter->f_present(tokenTextFeature)) /* Check for the presence of the particular feature */
      {
	 /* Here the feature valus is the text of the token */
	 Process_Token(tokenIter, tokenIter->S(tokenTextFeature).str(), wordList);
      }
      else
      {
	 Process_Token(tokenIter, tokenIter->name().str(), wordList);
      }

      /* Iterate over each word in the word list and insert them into the utterance structure */
      for(wordListIter = wordList.head(); wordListIter != 0; wordListIter = next(wordListIter))
      {
	 newWord = AddWord(u, wordList.item(wordListIter));	       

      /* add nil pos */
//      	 newWord->set("pos","nil");
	 append_daughter(tokenIter, newWord);
      
	 /* Add current word to the SylStructure relation */
	 sylStructure->append(newWord);
      }
   }
      
   return utt;
}

EST_Item * AddWord(EST_Utterance *u, EST_Item * t)
{
/* Add a word item to the end of Word relation */
   EST_Item *item = u->relation("Word")->append();

   item->set_name(t->name());    
	
   return item;
}

EST_Item * AddWord(EST_Utterance *u, EST_String &word)
{
   /* Add a word item to the end of Word relation */
   EST_Item *item = u->relation("Word")->append();

   item->set_name(word);    
	
   return item;
}

void Process_Token(EST_Item * nswToken, const char * token, EST_StrList& wordList)
{
   char * tokenType;
   char * nsw;
   int i,j;

   if(isNSW(nswToken->name().str()))
   {      
      Process_NSW(nswToken->name().str(), nswToken, token, wordList);
   }
   else
   {
      wordList.append(EST_String(token));
   }
}

bool isNSW(const char *token)
{
   if(strlen(token) != 1 && token[0]=='<') 
      return true;
   else 
      return false;
}

void Process_NSW(const char * tokenType, EST_Item * nswToken, const char * nsw, EST_StrList& wordList)
{
   int numVal=0, nyearVal=0, ntimeVal=0, ndateVal=0, ntelVal=0, nfloatVal=0, nfracVal=0, nratioVal=0, tokenVal=0;
   YY_BUFFER_STATE nswBuffer;

   printf("Token type: %s\n", tokenType);

   if(strcmp(tokenType,"<NYEAR>") == 0)
   {
      LISP     treeName;
      EST_Val  nswCategory;

      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      ntimeVal = yylex(SCAN_NTIME, strlen(nsw), wordList); // Temporarily changed to this.
      yy_delete_buffer(nswBuffer);

      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      nyearVal = yylex(SCAN_NYEAR, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);

      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      numVal = yylex(SCAN_NUM, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);

      printf("Num: %d, Year: %d, Time: %d\n", numVal,nyearVal,ntimeVal);
      
      /* To be continued from here */
      if(numVal && nyearVal && ntimeVal) /*** May not be complete logic: Panch check it ***/
      {

	 /* Get the tree name from the 'text_normalization_cart_tree' variable which is set through Scheme */
	 treeName = siod_get_lval("number_year_time_cart_tree","No text normalization cart tree");

	 /* Call the appropriate cart tree for token type disambiguation */
	 nswCategory = wagon_predict(nswToken, treeName);

	 /** nswCategory.string() returns EST_String **/

	 /********************************************************************************/
	 /*  Call Appropriate Token to Word Method for the disambiguated token category  */
	 /********************************************************************************/	 
	 printf("Identified as: %s\n", nswCategory.string().str());
	 if(strcmp(nswCategory.string().str(), "<NUM>") == 0)
	 {
	    Token_to_Words(nsw, numVal/100 -1, NUM, numVal%100, wordList);
	 }
	 else  if(strcmp(nswCategory.string().str(), "<NYEAR>") == 0)
	 {
	    Token_to_Words(nsw, nyearVal/100 -1, NYEAR, nyearVal%100, wordList);
	 }
	 else 
	 {
	    Token_to_Words(nsw, ntimeVal/100 -1, NTIME, ntimeVal%100, wordList);
	 }
      }
      else 
      {
	 treeName = siod_get_lval("number_year_cart_tree","No text normalization cart tree");
	 nswCategory = wagon_predict(nswToken, treeName);

	 if(strcmp(nswCategory.string().str(), "<NYEAR>") == 0)
	 {
	    Token_to_Words(nsw, nyearVal/100 -1, NYEAR, nyearVal%100, wordList);
	 }
	 else
	 {
	    Token_to_Words(nsw, numVal/100 -1, NUM, numVal%100, wordList);
	 }
	  
      }

      return;
   }

   if(strcmp(tokenType,"<NUM1>") == 0)
   {
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      numVal = yylex(SCAN_NUM, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);

      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      ntelVal = yylex(SCAN_NTEL, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);

      if(numVal && ntelVal)
      {
	 if(nsw[0] == '0') 
	 {
	    Token_to_Words(nsw, ntelVal/100 -1, NTEL, ntelVal%100, wordList);
	 }
	 else
	 {
	    /* model for Num-Ntel to be trained and used here */
	    /* currently using default NUM */
	    Token_to_Words(nsw, numVal/100 -1, NUM, numVal%100, wordList);
	 }
      }
      else
      {
	 Token_to_Words(nsw, numVal/100 -1, NUM, numVal%100, wordList);
      }

      return;
   }


   if(strcmp(tokenType,"<NFLOAT>") == 0)
   {

      LISP     treeName;
      EST_Val  nswCategory;
 
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      ntimeVal = yylex(SCAN_NTIME, strlen(nsw), wordList); // Temporarily changed to this.
      yy_delete_buffer(nswBuffer);

      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      ndateVal = yylex(SCAN_NDATE, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);

      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      nfloatVal = yylex(SCAN_NFLOAT, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);

      if(nfloatVal && ntimeVal && ndateVal)
      {
	 /* model for Nfloat-Time-Date to be trained and used; using default model "nfloat-time_cart_list" now */
	 treeName = siod_get_lval("float_time_cart_list","No text normalization cart list");
	 nswCategory = wagon_predict(nswToken, treeName);

	 if(strcmp(nswCategory.string().str(), "<NTIME>") == 0)
	 {
	    Token_to_Words(nsw, ntimeVal/100 -1, NTIME, ntimeVal%100, wordList);
	 }
	 else
	 {
	    Token_to_Words(nsw, nfloatVal/100 -1, NUM, nfloatVal%100, wordList);
	 }

      }
      else if(nfloatVal && ntimeVal)
      {
	 treeName = siod_get_lval("float_time_cart_list","No text normalization cart list");
	 nswCategory = wagon_predict(nswToken, treeName);

	 if(strcmp(nswCategory.string().str(), "<NTIME>") == 0)
	 {
	    Token_to_Words(nsw, ntimeVal/100 -1, NTIME, ntimeVal%100, wordList);
	 }
	 else
	 {
	    Token_to_Words(nsw, nfloatVal/100 -1, NUM, nfloatVal%100, wordList);
	 }

      }
      else if(nfloatVal && ndateVal)
      {
	 /* model for ndate-nfloat has to be trained and used; currently using deafult nfloat */
	 Token_to_Words(nsw, nfloatVal/100 -1, NUM, nfloatVal%100, wordList);
      }
      else /* only Nfloat */
      {
	 Token_to_Words(nsw, nfloatVal/100 -1, NUM, nfloatVal%100, wordList);
      }
      return;
   }

   if(strcmp(tokenType,"<NFRAC>") == 0)
   {
 
      LISP     treeName;
      EST_Val  nswCategory;
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      ndateVal = yylex(SCAN_NDATE, strlen(nsw), wordList); // Temporarily changed to this.
      yy_delete_buffer(nswBuffer);

      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      ntelVal = yylex(SCAN_NTEL, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);
 
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      nfracVal = yylex(SCAN_NFRAC, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);

      if(nfracVal && ndateVal)
      {
	 Token_to_Words(nsw, nfracVal/100 -1, NFRAC, nfracVal%100, wordList);
      }
      else if(nfracVal && ntelVal)
      {
	 Token_to_Words(nsw, nfracVal/100 -1, NFRAC, nfracVal%100, wordList);
      }
      else
      {
	 Token_to_Words(nsw, nfracVal/100 -1, NFRAC, nfracVal%100, wordList);
      }
      return;
   }

   if(strcmp(tokenType,"<NRATIO>") == 0)
   {
      LISP     treeName;
      EST_Val  nswCategory;
      
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      nratioVal = yylex(SCAN_NRATIO, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);

      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      ntimeVal = yylex(SCAN_NTIME, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);
      

      if(nfracVal && ntimeVal)
      {
	 /* model for Nratio-Ntime to be trained and used ; currently using default NRATIO */
	 Token_to_Words(nsw, nratioVal/100 -1, NRATIO, nratioVal%100, wordList);
      }
      else /* only nfrac */
      {
	 Token_to_Words(nsw, nratioVal/100 -1, NRATIO, nratioVal%100, wordList);
      }
      return;
   }

   if(strcmp(tokenType,"<NUM>") == 0)
   {
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      numVal = yylex(SCAN_NUM, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);

      Token_to_Words(nsw, numVal/100 -1, NUM, numVal%100, wordList);


      return;
   }

   
   if(strcmp(tokenType, "<NORD>") == 0)
   {
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      tokenVal = yylex(SCAN_NORD, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);
 
      Token_to_Words(nsw, tokenVal/100 -1, NORD, tokenVal%100, wordList);
   }

   if(strcmp(tokenType, "<NPRCT>") == 0)
   {
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      tokenVal = yylex(SCAN_NPRCT, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);
 
      Token_to_Words(nsw, tokenVal/100 -1, NPRCT, tokenVal%100, wordList);
   }

   if(strcmp(tokenType, "<NTIME>") == 0)
   {
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      tokenVal = yylex(SCAN_NTIME, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);
 
      Token_to_Words(nsw, tokenVal/100 -1, NTIME, tokenVal%100, wordList);
   }

   if(strcmp(tokenType, "<NDATE>") == 0)
   {
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      tokenVal = yylex(SCAN_NDATE, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);
 
      Token_to_Words(nsw, tokenVal/100 -1, NDATE, tokenVal%100, wordList);

      return;
   }

   if(strcmp(tokenType, "<NTEL>") == 0)
   {
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      tokenVal = yylex(SCAN_NTEL, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);
 
      printf("nsw = %s\n", nsw);
      Token_to_Words(nsw, tokenVal/100 -1, NTEL, tokenVal%100, wordList);

      return;
   }
   
   if(strcmp(tokenType, "<URL>") == 0)
   {
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      tokenVal = yylex(SCAN_NET, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);
 
      Token_to_Words(nsw, tokenVal/100 -1, URL, tokenVal%100, wordList);

      return;
   }

   if(strcmp(tokenType, "<EMAIL>") == 0)
   {
      nswBuffer = yy_scan_bytes(nsw, strlen(nsw));
      tokenVal = yylex(SCAN_NET, strlen(nsw), wordList);
      yy_delete_buffer(nswBuffer);
 
      Token_to_Words(nsw, tokenVal/100 -1, EMAIL, tokenVal%100, wordList);

      return;
   }

   if(strcmp(tokenType, "ALPHANUM") == 0)
   {
      Token_to_Words(nsw, tokenVal/100 -1, ALPHANUM, tokenVal%100, wordList);

      return;
   }
   //Added by sameer on 3/11/2004, thanks to krishna
   // Normalizer used to gobble up the commas used in phrasing
   // We go on to append commas in the word list 'just like that'
   if(strcmp(tokenType, "<PUNC>") == 0)
   {
	   Token_to_Words(nsw, 0, -1, 0, wordList);
	   return;
   }
}

